unit ListenerModule;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, SvcMgr, Dialogs,
  Forms, CHMnet, CHM;

type
  TLogEvent = procedure(Line: String) of object;

  TListenerModule = class
  public
    { Public declarations }
    OnLog: TLogEvent;
    constructor Create(NewAutoAckEngine: TCHMdynamicEngine;
      NewListener: TCHMllpServer);
    procedure Start();
    procedure Stop();
    procedure HandleClientConnected(Client: ICHMllpConnection);
    procedure HandleClientDisconnected(Client: ICHMllpConnection);
    procedure HandleClientErrorReceived(Client: ICHMllpConnection;
      ErrorMessage: String; ErrorCode: Integer);
    procedure HandleDataIgnored(Client: ICHMllpConnection; Data: String);
    procedure HandleErrorReceived(ErrorMessage: String; Code: Integer);
    procedure HandleMessageReceived(Client: ICHMllpConnection; Message: String);
  private
    { Private declarations }
    Listener: TCHMllpServer;
    AutoAckEngine: TCHMdynamicEngine;
    function PrettyPrintMessage(Message: string): string;
  end;

const
  APP_AUTOACK_VMD_FILE = 'autoack.vmd';
  APP_LISTEN_PORT = 5123;
  APP_MAX_MESSAGE_LOG_LENGTH = 2048; // Maximum size of message to display.

implementation

constructor TListenerModule.Create(NewAutoAckEngine: TCHMdynamicEngine;
  NewListener: TCHMllpServer);
begin
  AutoAckEngine := NewAutoAckEngine; 
  Listener := NewListener;
end;

procedure TListenerModule.Start();
begin
  try
    OnLog('Service Starting');

    AutoAckEngine.Load(ExtractFileDir(Application.ExeName) + '\' + APP_AUTOACK_VMD_FILE);
    Listener.StartListen(APP_LISTEN_PORT);

    OnLog('Service Started');
    OnLog('Listening on port ' + IntToStr(APP_LISTEN_PORT) + '.');
  except
    on E: Exception do
    begin
      OnLog('Error starting service: ' + E.Message);
    end;
  end;
end;

procedure TListenerModule.Stop();
begin
  try
    OnLog('Service Stopping');

    // Cleaning up after termination.
    Listener.StopListen();
    Listener.DisconnectAllClients();

    OnLog('Service Stopped');
  except
    on E: Exception do
    begin
      OnLog('Error stopping service: ' + E.Message);
    end;
  end;
end;

procedure TListenerModule.HandleClientConnected(Client: ICHMllpConnection);
begin
  OnLog('Client connected from: ' + Client.RemoteIpAddressAsString);
end;

procedure TListenerModule.HandleClientDisconnected(Client: ICHMllpConnection);
begin
  OnLog('Client disconnected from: ' + Client.RemoteIpAddressAsString);
end;

procedure TListenerModule.HandleClientErrorReceived(Client: ICHMllpConnection;
  ErrorMessage: String; ErrorCode: Integer);
begin
  OnLog('Client error received: ' + PrettyPrintMessage(ErrorMessage));
end;

procedure TListenerModule.HandleDataIgnored(Client: ICHMllpConnection;
  Data: String);
begin
  OnLog('Ignored data outside of LLP protocol received from: ' + Client.RemoteIpAddressAsString);
  OnLog(PrettyPrintMessage(Data));
end;

procedure TListenerModule.HandleErrorReceived(ErrorMessage: String;
  Code: Integer);
begin
  OnLog('Error received: ' + PrettyPrintMessage(ErrorMessage));
end;

procedure TListenerModule.HandleMessageReceived(Client: ICHMllpConnection;
  Message: String);
var
  AckMessage: string;
begin
  OnLog('Received a message from ' + Client.RemoteIpAddressAsString + ':');
  OnLog(PrettyPrintMessage(Message));
  OnLog('Sending ACK to ' + Client.RemoteIpAddressAsString);

  AckMessage := '';

  try
    try
      // We create an acknowledgment message to send back to the client
      // using the auto-acknowledgement technique.
      AckMessage := AutoAckEngine.GenerateAck(Message);
    except
      on E: Exception do
      begin
        OnLog('Error generating Ack: ' + E.Message);
      end;
    end;
  finally
    if AckMessage = '' then
    begin
      OnLog('Generating Ack failed! Sending a hard-coded Ack.');
      AckMessage := 'MSH|^~\&|ACME|ACME_APP|XYZ|XYZ_APP|||NACK|XXX|P|2.4|' + Chr(10) + 'MSA|AA|XXX|' + Chr(10);
    end;

    // We send the ACK message.
    OnLog(PrettyPrintMessage(AckMessage));
    Client.TransmitMessage(AckMessage);
  end;
end;

// This routine converts \r --> \r\n and truncates the message if necessary
// so that HL7 messages can be seen in the log.
function TListenerModule.PrettyPrintMessage(Message: string): string;
begin
  Result := Listener.MakeMessagePrintable(Message, #13#10, APP_MAX_MESSAGE_LOG_LENGTH);
end;

end.

